@@ -11,6 +11,7 @@ ualter.junior@gmail.com
 
 import os, sys, configparser, logging, re
 import xml.etree.ElementTree as ET
+from pathlib import Path
 from urllib.request import pathname2url
 from tinydb import TinyDB, Query, where
 from tinydb.storages import JSONStorage
@@ -31,10 +32,11 @@ _logger.setLevel(logging.INFO)
 
 GIT = "git"
 MAVEN = "maven"
-USER_DIR = expanduser("~") + "/.pymvngit/"
+NAME_DB_MACROS = "macros.json"
+USER_DIR = os.path.join(expanduser("~"),".pymvngit")
 LOG_DIR  = os.path.join(USER_DIR,"logs")
 DB_PATH_PROJECT = os.path.join(USER_DIR,"#.json")
-DB_PATH_MACRO  = os.path.join(USER_DIR,"macros.json")
+DB_PATH_MACRO  = os.path.join(USER_DIR,NAME_DB_MACROS)
 INI_FILE = os.path.join(USER_DIR,"pymvngit.ini")
 
 class PyMvnGit:
@@ -162,14 +164,13 @@ class PyMvnGit:
                print("     Ok, bye!\n")
                sys.exit()    
         else:       
-           self._connectDBs()       
+           self._connectDBs(startDatabases=True)       
 
     def _connectDBs(self, startDatabases=False):
         if not os.path.exists(self._fileDBProject) and not startDatabases:
            print("")
-           Console.error("{green}There's no repository created for {iblue}{curdir}{green}, please start it with the command: {iblue}pymvngit start{reset}". \
+           Console.error("{green}There's no repository created for {iblue}{curdir}{green}, please start it using: {iblue}pymvngit start{reset}". \
                          format(curdir=self._fileDBProject,iblue=Colors.IBLUE,igreen=Colors.IGREEN,green=Colors.GREEN,reset=Colors.RESET))
-           print("")              
            sys.exit()              
 
         self._databaseProject = TinyDB(self._fileDBProject, indent=4)
@@ -178,15 +179,19 @@ class PyMvnGit:
         self._initDBMacroDefault()
 
     def _currentDirEncoded(self):
-        nameCurrentDirEncoded = pathname2url(self._currentDir()).replace(":","drive_")
+        nameCurrentDirEncoded = self._encodeDirectoryName(self._currentDir()).replace(":","drive_")
         logging.info("nameCurrentDirEncoded=" + nameCurrentDirEncoded)
         return nameCurrentDirEncoded
     def _currentDir(self):
         currentDirectory = os.getcwd()
         #For testing/development
         #currentDirectory  = "/home/ualter/Developer/teachstore"
+        currentDirectory = "d:\\Developer\\code\\teachstore"
         return currentDirectory
 
+    def _encodeDirectoryName(self, directory):
+        return directory.replace("\\","_").replace("/","_").replace(":","_drive_").lower()
+
     def listMacros(self, summary=False, jsonFormat=False):
         """
         \033[0;32mList all the macros available\033[0;0m
@@ -266,26 +271,47 @@ class PyMvnGit:
         \033[0;0m   
         """
 
-        if not self._connectedDatabase:
-           self._connectDBs()
+        for path in Path(USER_DIR).rglob('*.json'):
+            if path.name != NAME_DB_MACROS:
+               db      = TinyDB(path, indent=4)   
+               records = db.all()
+               if jsonFormat:
+                  Utils.printJSON(Utils.dictToJSON(records))
+               else:
+                  tableArgs   = TableArgs()
+                  header      = ["#","Name","Path","Maven","Git"]
+                  prettyTable = PrettyTable(header)
+                  idx_lin     = 1
+                  for r in records:
+                      columns  = [str(idx_lin), r["name"], r["path"], r["maven"], r["git"]]
+                      prettyTable.addRow(columns)
+                      idx_lin += 1
+       
+                  prettyTable.sortByColumn(0)   
+                  prettyTable.numberSeparator = True
+                  prettyTable.ascendingOrder(True)
+                  nameDecoded = path.name.replace("_","/").replace("//","/").replace("/drive",":")
+                  Console.printTable("FOLDER/REPOSITORY: {}{}{}".format(Colors.IGREEN,nameDecoded,Colors.GREEN), prettyTable, tableArgs)    
+               db.close()   
 
-        records = self._databaseProject.all()
-        if jsonFormat:
-           Utils.printJSON(Utils.dictToJSON(records))
-        else:
-           tableArgs   = TableArgs()
-           header      = ["#","Name","Path","Maven","Git"]
-           prettyTable = PrettyTable(header)
-           idx_lin     = 1
-           for r in records:
-               columns  = [str(idx_lin), r["name"], r["path"], r["maven"], r["git"]]
-               prettyTable.addRow(columns)
-               idx_lin += 1
+    def clone(self, repositories, localPath, branchName,isRepoListFile):
+        args = []
+        args.append(repositories)
+        args.append(localPath)
+        args.append(branchName)
+        args.append(isRepoListFile)
+        pygit = PyGit("clone",args,None) 
+        pygit.execute()
 
-           prettyTable.sortByColumn(0)   
-           prettyTable.numberSeparator = True
-           prettyTable.ascendingOrder(True)
-           Console.printTable("PROJECTS: {}{}{}".format(Colors.IGREEN,self._currentDir(),Colors.GREEN), prettyTable, tableArgs)    
+        # Create Repository for the Clone
+        self._fileDBProject = DB_PATH_PROJECT.replace("#",self._encodeDirectoryName(localPath))
+        self._connectDBs(True)
+        if pygit.response:
+           for r in pygit.response:
+               exist = False if len(self._databaseProject.search(where('name') == r["name"])) == 0 else True
+               # Do not include duplicated (most likely the folder was delete but the database entry not)
+               if not exist:
+                  self._createModule(r["name"],r["path"],r["git"],r["maven"])
 
     def start(self):
         """
@@ -474,17 +500,48 @@ def main():
     elif "editMacros".upper() == command.upper() or "editMacro".upper() == command.upper() or "emac".upper() == command.upper():
        pymvngit.editMacros()   
     elif "listMacros".upper() == command.upper():
-       summary = False
-       if len(sys.argv) > 2 and sys.argv[2] == "summary":
-          summary = True
+       summary = True
+       if len(sys.argv) > 2 and sys.argv[2] == "-v":
+          summary = False
        pymvngit.listMacros(summary)
-    elif "start".upper() == command.upper():   
+    elif "start".upper() == command.upper():
        pymvngit.start()
+    elif "clone".upper() == command.upper():
+       if len(sys.argv) < 6:
+          Console.error("Missing parameters! Please, check the syntax of the command clone!")
+          syntax(clearScreen=False)
+          sys.exit()
+       elif len(sys.argv) == 7:
+          # Must have 6 or 8 parameters 
+          Console.error("Missing parameters! Please, check the syntax of the command clone!")
+          syntax(clearScreen=False)
+          sys.exit() 
+       else:
+          repositories   = []
+          localPath      = ""
+          branchName     = "master" 
+          index          = 0
+          isRepoListFile = False
+          for a in sys.argv:
+              index += 1
+              if a == "-r":
+                 for repo in sys.argv[index].split(","):
+                     repositories.append(repo.strip())
+              elif a == "-p":
+                 localPath = sys.argv[index].strip()
+              elif a == "-f":
+                 repositories.append(sys.argv[index].strip())
+                 isRepoListFile = True
+              elif a == "-b":
+                 branchName = sys.argv[index].strip()   
+
+          pymvngit.clone(repositories,localPath,branchName,isRepoListFile)       
     else:   
        pymvngit.executeMacro(command)
 
-def syntax():
-    Console.clear()
+def syntax(clearScreen=True):
+    if clearScreen:
+       Console.clear()
     Console.printHeaderCommand("PyGitMvn Tool")    
     print("""
         {green}Tool to manage a group of Git/Maven projects.
@@ -499,12 +556,19 @@ def syntax():
 
         {iblue}FUNCTIONS:{green}
         - start...........: {blue}Create a repository from the current directory. Every subfolder will be 
-                            analyzed and if it's a Git/Maven project, will be part of this created repository. 
+                            analyzed and if it's a Git/Maven project, will be part of this created repository.{green}
+        - clone...........: {blue}Create a repository cloning all the URL repository(ies) passed as argument
+                            {green}#{cyan}arguments:
+                            - Repositories.......: {cyan}(1){green} -r "https://github/projectA, https://github/projectB"{green}
+                                                      {cyan}or{green}
+                                                   {cyan}(2){green} -f file-name.txt     {cyan}# file with the list of repositories
+                            - Local path.........: {green}-p "/home/my/projects"{cyan}
+                            - Branch (optional)..: {green}-b "develop"{cyan}             #"branch" is the default value
         - {cyan}[{green}macro{cyan}]{green}.........: {blue}Name of a macro to be executed.{green}
         - listProjects....: {blue}List the projects part of the repository associated with the current directory.{green}
         - listMacros......: {blue}List the macros configured. 
                             {green}#{cyan}arguments:
-                            - {green}{{summary}}{cyan}: list a summary of all commands on a single table
+                            - Show more details, print a table for each one of them..: {green}{{-v}}{cyan}
         {iblue}SYNTAX:{green}
           pymvngit {iblue}[MACRO] {cyan}{{arguments}}
         {iblue}EXAMPLES:{green}
